@@ -88,6 +88,9 @@ type Options struct {
 	MulticastAddress string
 	ShowQrCode       bool
 	Exclude          []string
+	Quiet            bool
+	DisableClipboard bool
+	ExtendedClipboard bool
 }
 
 type SimpleMessage struct {
@@ -197,6 +200,14 @@ func New(ops Options) (c *Client, err error) {
 	c.Options = ops
 	Debug(c.Options.Debug)
 
+	// redirect stderr to null if quiet mode is enabled
+	if c.Options.Quiet {
+		devNull, err := os.OpenFile(os.DevNull, os.O_WRONLY, 0)
+		if err == nil {
+			os.Stderr = devNull
+		}
+	}
+
 	if len(c.Options.SharedSecret) < 6 {
 		err = fmt.Errorf("code is too short")
 		return
@@ -258,6 +269,7 @@ type TransferOptions struct {
 	KeepPathInRemote bool
 }
 
+// helper function checking for an empty folder
 func isEmptyFolder(folderPath string) (bool, error) {
 	f, err := os.Open(folderPath)
 	if err != nil {
@@ -313,15 +325,6 @@ func isChild(parentPath, childPath string) bool {
 	return !strings.HasPrefix(relPath, "..")
 }
 
-func recursiveFiles(path string) (paths []string, err error) {
-	paths = []string{strings.ToLower(path)}
-	err = filepath.Walk(path, func(path string, info os.FileInfo, err error) error {
-		paths = append(paths, strings.ToLower(path))
-		return nil
-	})
-	return
-}
-
 // This function retrieves the important file information
 // for every file that will be transferred
 func GetFilesInfo(fnames []string, zipfolder bool, ignoreGit bool, exclusions []string) (filesInfo []FileInfo, emptyFolders []FileInfo, totalNumberFolders int, err error) {
@@ -671,7 +674,13 @@ On the other computer run:
 (For Linux/macOS)
     CROC_SECRET=%[1]q croc %[2]s
 `, c.Options.SharedSecret, flags.String())
-	copyToClipboard(c.Options.SharedSecret)
+	if !c.Options.DisableClipboard {
+		clipboardText := c.Options.SharedSecret
+		if c.Options.ExtendedClipboard {
+			clipboardText = fmt.Sprintf("CROC_SECRET=%q croc %s", c.Options.SharedSecret, strings.TrimSpace(flags.String()))
+		}
+		copyToClipboard(clipboardText, c.Options.Quiet, c.Options.ExtendedClipboard)
+	}
 	if c.Options.ShowQrCode {
 		showReceiveCommandQrCode(fmt.Sprintf("%[1]s", c.Options.SharedSecret))
 	}
@@ -679,7 +688,7 @@ On the other computer run:
 		machid, _ := machineid.ID()
 		fmt.Fprintf(os.Stderr, "\rYour machine ID is '%s'\n", machid)
 	}
-	// // c.spinner.Suffix = " waiting for recipient..."
+	// c.spinner.Suffix = " waiting for recipient..."
 	// c.spinner.Start()
 	// create channel for quitting
 	// connect to the relay for messaging
@@ -753,6 +762,12 @@ On the other computer run:
 					dataDecrypt, decryptErr = crypt.Decrypt(data, kB)
 					if decryptErr != nil {
 						log.Tracef("error decrypting: %v: '%s'", decryptErr, data)
+						// relay sent a messag encrypted with an invalid key.
+						// consider this a security issue and abort
+						if strings.Contains(decryptErr.Error(), "message authentication failed") {
+							errchan <- decryptErr
+							return
+						}
 					} else {
 						// copy dataDecrypt to data
 						data = dataDecrypt
@@ -830,8 +845,7 @@ On the other computer run:
 
 	err = <-errchan
 	if err == nil {
-		// return if no error
-		return
+		return // no error
 	} else {
 		log.Debugf("error from errchan: %v", err)
 		if strings.Contains(err.Error(), "could not secure channel") {
@@ -839,7 +853,7 @@ On the other computer run:
 		}
 	}
 	if !c.Options.DisableLocal {
-		if strings.Contains(err.Error(), "refusing files") || strings.Contains(err.Error(), "EOF") || strings.Contains(err.Error(), "bad password") {
+		if strings.Contains(err.Error(), "refusing files") || strings.Contains(err.Error(), "EOF") || strings.Contains(err.Error(), "bad password") || strings.Contains(err.Error(), "message authentication failed") {
 			errchan <- err
 		}
 		err = <-errchan
@@ -988,7 +1002,6 @@ func (c *Client) Receive() (err error) {
 	if c.Options.TestFlag || (!usingLocal && !c.Options.DisableLocal && !isIPset) {
 		// ask the sender for their local ips and port
 		// and try to connect to them
-
 		var ips []string
 		err = func() (err error) {
 			var A *pake.Pake
@@ -1179,7 +1192,7 @@ func (c *Client) transfer() (err error) {
 		}
 	}
 
-	if c.Options.Stdout && !c.Options.IsSender {
+	if c.Options.Stdout && !c.Options.IsSender && len(c.FilesToTransfer) > 0 && c.FilesToTransferCurrentNum < len(c.FilesToTransfer) {
 		pathToFile := path.Join(
 			c.FilesToTransfer[c.FilesToTransferCurrentNum].FolderRemote,
 			c.FilesToTransfer[c.FilesToTransferCurrentNum].Name,
@@ -1291,7 +1304,7 @@ func (c *Client) processMessageFileInfo(m message.Message) (done bool, err error
 			}
 		}
 	}
-	// // check the totalSize does not exceed disk space
+	// check the totalSize does not exceed disk space
 	// usage := diskusage.NewDiskUsage(".")
 	// if usage.Available() < uint64(totalSize) {
 	// 	return true, fmt.Errorf("not enough disk space")
@@ -1455,7 +1468,6 @@ func (c *Client) processMessagePake(m message.Message) (err error) {
 		}(i)
 	}
 	wg.Wait()
-
 	if !c.Options.IsSender {
 		log.Debug("sending external IP")
 		err = message.Send(c.conn[0], c.Key, message.Message{
@@ -1849,6 +1861,14 @@ func (c *Client) updateIfRecipientHasFileInfo() (err error) {
 			}
 		} else {
 			log.Debugf("hashes are equal %x == %x", fileHash, fileInfo.Hash)
+
+			if !fileInfo.ModTime.IsZero() {
+				if err := os.Chtimes(path.Join(fileInfo.FolderRemote, fileInfo.Name), fileInfo.ModTime, fileInfo.ModTime); err != nil {
+					log.Warnf("chtimes %v: %v", fileInfo.ModTime, err)
+				} else {
+					log.Debugf("chtimes %v", fileInfo.ModTime)
+				}
+			}
 		}
 		if errHash != nil {
 			// probably can't find, its okay
@@ -2135,26 +2155,57 @@ func (c *Client) sendData(i int) {
 	}
 }
 
-func copyToClipboard(str string) {
+// isExecutableInPath checks for the availability of an executable
+func isExecutableInPath(executableName string) bool {
+	_, err := exec.LookPath(executableName)
+	return err == nil
+}
+
+// copyToClipboard tries to send the code to the operating system clipboard
+func copyToClipboard(str string, quiet bool, extendedClipboard bool) {
 	var cmd *exec.Cmd
 	switch runtime.GOOS {
+	// Windows should always have clip.exe in PATH by default
 	case "windows":
 		cmd = exec.Command("clip")
+	// MacOS uses pbcopy
 	case "darwin":
 		cmd = exec.Command("pbcopy")
-	case "linux":
-		if os.Getenv("XDG_SESSION_TYPE") == "wayland" {
-			cmd = exec.Command("wl-copy")
-		} else {
-			cmd = exec.Command("xclip", "-selection", "clipboard")
+	// These Unix-like systems are likely using Xorg(with xclip or xsel) or Wayland(with wl-copy or waycopy)
+	case "linux", "android", "hurd", "freebsd", "openbsd", "netbsd", "dragonfly", "solaris", "illumos", "plan9":
+		if os.Getenv("XDG_SESSION_TYPE") == "wayland" { // Wayland running
+			if isExecutableInPath("wl-copy") {
+				cmd = exec.Command("wl-copy")
+			} else if isExecutableInPath("waycopy") {
+				cmd = exec.Command("waycopy")
+			}
+		} else if os.Getenv("XDG_SESSION_TYPE") == "x11" || os.Getenv("XDG_SESSION_TYPE") == "xorg" { // Xorg running
+			if isExecutableInPath("xclip") {
+				cmd = exec.Command("xclip", "-selection", "clipboard")
+			}
+		} else if isExecutableInPath("xsel") {
+			cmd = exec.Command("xsel", "-b")
+		} else if isExecutableInPath("termux-clipboard-set") {
+			cmd = exec.Command("termux-clipboard-set")
 		}
 	default:
 		return
 	}
+	// Nothing has been found
+	if cmd == nil {
+		return
+	}
+	// Sending stdin into the available clipboard program
 	cmd.Stdin = bytes.NewReader([]byte(str))
 	if err := cmd.Run(); err != nil {
 		log.Debugf("error copying to clipboard: %v", err)
 		return
 	}
-	fmt.Fprintf(os.Stderr, "Code copied to clipboard\n")
+	if !quiet {
+		if extendedClipboard {
+			fmt.Fprintf(os.Stderr, "Command copied to clipboard!\n")
+		} else {
+			fmt.Fprintf(os.Stderr, "Code copied to clipboard!\n")
+		}
+	}
 }
